package nl.xelion.xcc.api.examples.client;

import java.lang.reflect.Type;
import java.util.Objects;

import nl.xelion.xcc.api.examples.client.http.HTTPConstants.HTTPMethod;

/**
 * This class hold attributes of a REST request. The method and path combination identifies the resource. The type field
 * can be fed to a JSON parser to build the response object.
 * <p>
 * This software is provided under the MIT <a href="https://opensource.org/licenses/MIT">license</a>
 */
public class Request<T>
{
    /**
     * the HTTP method
     */
    public final HTTPMethod method;

    /**
     * the resource path
     */
    public final String path;

    /**
     * the query path of the URL
     */
    public final String query;

    /**
     * an optional response object type.
     */
    public final Type responseObjectType;

    public Request(HTTPMethod method, String path)
    {
        this(method, path, null);
    }

    public Request(HTTPMethod method, String path, Type responseObjectType)
    {
        this(method, path, responseObjectType, null);
    }

    public Request(HTTPMethod method, String path, Type responseObjectType, String query)
    {
        this.method = Objects.requireNonNull(method, "HTTP method");
        this.path = Objects.requireNonNull(path, "path");
        this.query = query;
        this.responseObjectType = responseObjectType;
    }

    public HTTPMethod getMethod()
    {
        return method;
    }

    public String getPath()
    {
        return query == null ? path : path + "?" + query;
    }

    public Type getResponseObjectType()
    {
        return responseObjectType;
    }

    @Override
    public String toString()
    {
        String s = getMethod().name() + " " + getPath();

        if (responseObjectType != null)
        {
            return s + " -> " + responseObjectType.toString();
        }

        return s;
    }

    /**
     * Set the query part of the URL.
     * 
     * @return a new request with the same attributes save the query.
     */
    public Request<T> withQuery(String newQuery)
    {
        return new Request<>(this.method, this.path, this.responseObjectType, newQuery);
    }
}
