package nl.xelion.xcc.api.examples.client;

import java.lang.reflect.Type;
import java.util.Objects;

import nl.xelion.xcc.api.examples.client.datamodel.RESTList;
import nl.xelion.xcc.api.examples.client.datamodel.RESTObject;
import nl.xelion.xcc.api.examples.client.http.HTTPConstants.HTTPMethod;

/**
 * A REST resource
 * <p>
 * This software is provided under the MIT <a href="https://opensource.org/licenses/MIT">license</a>
 */
public class Resource<T>
{
    /**
     * the resource path
     */
    private final String path;

    /**
     * an optional response object type. An example of a request of this type: GET communication/123 -> RESTObject
     * <communication>
     */
    public final Type objectResponseType;

    /**
     * an optional list response type. An example of a request of this type: GET communication -> RESTList
     * <communication>
     */
    public final Type listResponseType;

    public Resource(String path, Type objectResponseType)
    {
        this(path, objectResponseType, null);
    }

    public Resource(String path, Type objectResponseType, Type listResponseType)
    {
        this.path = Objects.requireNonNull(path);
        this.objectResponseType = objectResponseType;
        this.listResponseType = listResponseType;
    }

    public String path()
    {
        return path;
    }

    public Type objectResponseType()
    {
        return objectResponseType;
    }

    public Type listResponseType()
    {
        return listResponseType;
    }

    /**
     * Create a new resource based on this resource where the path is extended with the specified resource id. For
     * example:
     * 
     * <pre>
     *     Resource communication = 
     *     Email email = communication.withID(...);
     * </pre>
     * 
     * @param resourceID
     *            the resource id.
     * @return a resource with the same type as this resource and with the path:
     *         <code>this.path + "/" + resourceID</code>
     */
    public Resource<T> withID(String resourceID)
    {
        return new Resource<>(
            (path.endsWith("/") ? path : path + "/") + resourceID,
            objectResponseType,
            listResponseType);
    }

    @Override
    public String toString()
    {
        return path + " : " + objectResponseType + (listResponseType == null ? "" : " list: " + listResponseType);
    }

    /*
     * The following methods create requests from this resource.
     */

    /**
     * Create a get request for the resource.
     * <p>
     * The GSON class TokenType is used to retrieve the dynamic type of a Java generic.
     */
    public Request<RESTObject<T>> get()
    {
        return new Request<>(HTTPMethod.GET, path, objectResponseType);
    }

    /**
     * Create a request for this resource.
     */
    public Request<RESTList<T>> list()
    {
        return new Request<>(HTTPMethod.GET, path, listResponseType);
    }

    /**
     * Create a put request for the resource.
     */
    public Request<RESTObject<T>> put()
    {
        return new Request<>(HTTPMethod.PUT, path, objectResponseType);
    }

    /**
     * Create a post request for the resource, usually to creates a new resource.
     */
    public Request<RESTObject<T>> post()
    {
        return new Request<>(HTTPMethod.POST, path, objectResponseType);
    }

    /**
     * Create a delete request for the resource to remove the resource.
     */
    public Request<Void> delete()
    {
        return new Request<>(HTTPMethod.DELETE, path, null);
    }

    /**
     * Create a post request for a resource that doesn't wrap the return value in a RESTObject (action like resource).
     */
    public Request<T> exec()
    {
        return new Request<>(HTTPMethod.POST, path, objectResponseType);
    }

    /**
     * Create a put request for the resource that returns nothing. These requests often set a global parameter.
     */
    public Request<Void> set()
    {
        return new Request<>(HTTPMethod.PUT, path, objectResponseType);
    }
}
