package nl.xelion.xcc.api.examples.client;

import java.util.Objects;
import java.util.Optional;
import java.util.function.Consumer;
import java.util.function.Function;

import nl.xelion.xcc.api.examples.client.datamodel.ErrorResponse;

/**
 * This class holds either a Xelion response value of the type T, or an error response.
 * <p>
 * The response value may be null, the error response may not. A useful extension for this class is to store the value
 * in an {@link Optional}. 
 * <p>
 * This software is provided under the MIT <a href="https://opensource.org/licenses/MIT">license</a>
 */
public class Response<T>
{
    private final T value;
    private final ErrorResponse error;
    
    private Response(final T value, final ErrorResponse error)
    {
        this.value = value;
        this.error = error;
    }

    public static <T> Response<T> ofValue(final T value)
    {
        return new Response<>(value, null);
    }

    public static <T> Response<T> ofError(final ErrorResponse error)
    {
        return new Response<>(null, Objects.requireNonNull(error));
    }

    /**
     * @return the value
     */
    public T value()
    {
        return value;
    }

    /**
     * @return the error
     */
    public ErrorResponse error()
    {
        return error;
    }

    /**
     * @return true if it is not an error response
     */
    public boolean success()
    {
        return error == null;
    }

    /**
     * Transforms the value if set.
     * 
     * @param f
     *            the transformation
     * @return the transformed object.
     */
    public <U> Response<U> map(Function<? super T, U> f)
    {
        if (success())
        {
            return ofValue(f.apply(value));
        }
        else
        {
            return ofError(error);
        }
    }

    /**
     * Consume the value.
     * 
     * @param f
     *            the value consumer
     */
    public void apply(Consumer<? super T> f)
    {
        if (success())
        {
            f.accept(value);
        }
    }

    /**
     * Consume the error.
     * 
     * @param f
     *            the error consumer
     */
    public void applyToError(Consumer<ErrorResponse> f)
    {
        if (!success())
        {
            f.accept(error);
        }
    }

    @Override
    public String toString()
    {
        return success() ? (value == null ? "[]" : value.toString()) : error.toString();
    }
}
