package nl.xelion.xcc.api.examples.client;

import java.io.IOException;
import java.net.HttpURLConnection;
import java.util.Objects;

import nl.xelion.xcc.api.examples.client.datamodel.ErrorResponse;
import nl.xelion.xcc.api.examples.client.datamodel.LoginCredentials;
import nl.xelion.xcc.api.examples.client.datamodel.LoginResponse;
import nl.xelion.xcc.api.examples.client.http.HTTPConstants.HTTPHeader;
import nl.xelion.xcc.api.examples.client.http.HTTPRequest;
import nl.xelion.xcc.api.examples.client.http.HTTPRequestBuilder;
import nl.xelion.xcc.api.examples.client.http.HTTPResponse;

/**
 * This class creates an Xelion session and handles Xelion HTTP requests.
 * <p>
 * One the things that are left out here renewing the session token.
 * <p>
 * This software is provided under the MIT <a href="https://opensource.org/licenses/MIT">license</a>
 */
public class Session
{
    /**
     * The session authorization token. This token must be specified in the authorization header of each request.
     * <p>
     * The token must be refreshed periodically. This task is not implemented here.
     */
    private String sessionToken;

    /**
     * The API's base URL.
     */
    private final String baseURL;

    public Session(String baseURL, String sessionToken)
    {
        this.baseURL = Objects.requireNonNull(baseURL);
        this.sessionToken = Objects.requireNonNull(sessionToken);
    }

    /**
     * Login to Xelion and starts a new session.
     * 
     * @param credentials
     *            the user credentials
     * @param host
     *            the host of the Xelion API
     * @param tenant
     *            the tenant name, or null if master tenant is used.
     * @return the created session.
     * @throws RequestException
     *             on failed login
     */
    public static Response<Session> login(LoginCredentials credentials, String host, String tenant)
        throws RequestException
    {
        String baseURL = baseURL(host, tenant);
        Response<LoginResponse> response = request(Resources.login.exec(), baseURL, null, credentials);
        return response.map(o -> {
            return new Session(baseURL, o.getAuthentication());
        });
    }

    /**
     * This method requests an Xelion resource.
     * 
     * @param request
     *            the Xelion resource information
     * @return the resource's contents or the error response on error
     * @throws RequestException
     *             on failed request
     */
    public <T> Response<T> request(Request<T> request) throws RequestException
    {
        return request(request, baseURL, sessionToken, null);
    }

    /**
     * This method requests an Xelion resource and sends an object in the body of the request.
     * 
     * @param request
     *            the Xelion resource information
     * @param object
     *            an optional object to send with the request.
     * @return the resource's contents or the error response on error
     * @throws RequestException
     *             on failed request
     */
    public <T> Response<T> request(Request<T> request, Object object) throws RequestException
    {
        return request(request, baseURL, sessionToken, object);
    }

    @Override
    public String toString()
    {
        return "Session on " + baseURL;
    }

    /**
     * Requests an Xelion resource
     */
    private static <T> Response<T> request(Request<T> request, String baseURL, String token, Object object)
        throws RequestException
    {
        try
        {
            HTTPResponse response = httpRequest(request, baseURL, token, object).invoke();
            ErrorResponse error = detectError(response);
            if (error != null)
            {
                return Response.ofError(error);
            }
            else if (request.getResponseObjectType() == null)
            {
                return Response.ofValue(null);
            }
            else
            {
                return Response.ofValue(response.getJSONContent(request.getResponseObjectType()));
            }
        }
        catch (IOException e)
        {
            throw new RequestException(e.getMessage(), e);
        }
    }

    /**
     * @return an HTTP request for the REST request, with the specified authorization token and object.
     */
    private static HTTPRequest httpRequest(Request<?> request, String baseURL, String token, Object object)
    {
        HTTPRequestBuilder builder = new HTTPRequestBuilder(baseURL + "/" + request.getPath());
        builder.setMethod(request.getMethod());
        if (token != null)
        {
            builder.setHeader(HTTPHeader.Authorization, "xelion " + token);
        }
        if (object != null)
        {
            builder.setJSONContent(object);
        }
        return builder.build();
    }

    /**
     * This method return the error response if error occurred. In that case the HTTP status code is not 200 (HTTP-OK).
     */
    private static ErrorResponse detectError(HTTPResponse response) throws RequestException
    {
        if (response.getStatus() != HttpURLConnection.HTTP_OK)
        {
            try
            {
                ErrorResponse error = response.getJSONContent(ErrorResponse.class);
                if (error != null && error.getErrorCode() > 0)
                {
                    return error;
                }
            }
            catch (Exception e)
            {
            }
            throw new RequestException("HTTP status: " + response.getStatus() + ". No error info sent.");
        }

        return null;
    }

    /**
     * this method builds the API's base URL that has the following format: https//[host]/api/v1/[tenant]
     */
    private static final String baseURL(String host, String tenant)
    {
        if (!host.startsWith("http"))
        {
            host = "https://" + host;
        }

        if (!host.endsWith("/"))
        {
            host += "/";
        }

        if (tenant == null || tenant.isEmpty())
        {
            tenant = "master";
        }

        return host + "api/v1/" + tenant;
    }
}
