package nl.xelion.xcc.api.examples.client.datamodel;

import java.util.List;

/**
 * A response object wraps a value of a resource. This object is returned by resource requests that generate a single
 * value.
 * <p>
 * In addition to the wrapped value, this object contains a list of HATEOAS links.
 * <p>
 * This software is provided under the MIT <a href="https://opensource.org/licenses/MIT">license</a>
 * 
 * @author arnon
 */
public class RESTList<T>
{
    /**
     * The response objects.
     */
    private final List<RESTObject<T>> data;

    /**
     * Paging information and HATEOAS links.
     */
    private final MetaData meta;

    public RESTList(List<RESTObject<T>> data)
    {
        this(data, (MetaData) null);
    }

    public RESTList(List<RESTObject<T>> data, MetaData meta)
    {
        this.data = data;
        this.meta = meta;
    }

    public RESTList(List<RESTObject<T>> data, Paging paging)
    {
        this(data, new MetaData(paging));
    }

    /**
     * @return the data
     */
    public List<RESTObject<T>> getData()
    {
        return data;
    }

    /**
     * @return the meta
     */
    public MetaData getMeta()
    {
        return meta;
    }

    @Override
    public String toString()
    {
        return "[data=" + (data != null ? data.subList(0, Math.min(data.size(), 4)) : null) + ", meta=" + meta + "]";
    }

    /**
     * The list meta data
     */
    public static class MetaData
    {
        /**
         * HATEOAS links of the list.
         */
        private final List<HATEOASLink> links;

        /**
         * Paging information.
         */
        private final Paging paging;

        public MetaData(Paging paging)
        {
            this(paging, null);
        }

        public MetaData(Paging paging, List<HATEOASLink> links)
        {
            this.paging = paging;
            this.links = links;
        }

        /**
         * @return the paging
         */
        public Paging getPaging()
        {
            return paging;
        }

        /**
         * @return the links
         */
        public List<HATEOASLink> getLinks()
        {
            return links;
        }

        @Override
        public String toString()
        {
            return "[" + (paging != null ? "paging=" + paging : "") + "]";
        }
    }

    /**
     * paging information.
     */
    public static class Paging
    {
        /**
         * Cursor to the beginning of the page.
         */
        private final String previousId;

        /**
         * Cursor to the end of the page.
         */
        private final String nextId;

        public Paging(String previous, String next)
        {
            this.previousId = previous;
            this.nextId = next;
        }

        /**
         * @return the before
         */
        public String getPreviousID()
        {
            return previousId;
        }

        /**
         * @return the after
         */
        public String getNextID()
        {
            return nextId;
        }

        @Override
        public String toString()
        {
            return "Paging [previous=" + previousId + ", next=" + nextId + "]";
        }
    }
}
