package nl.xelion.xcc.api.examples.client.http;

import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Map;
import java.util.Objects;

import nl.xelion.xcc.api.examples.client.http.HTTPConstants.HTTPHeader;
import nl.xelion.xcc.api.examples.client.http.HTTPConstants.HTTPMethod;

/**
 * An HTTP request.
 * <p>
 * This software is provided under the MIT <a href="https://opensource.org/licenses/MIT">license</a>
 */
public class HTTPRequest
{
    private final String url;
    private final HTTPMethod method;
    private final Map<String, String> headers;
    private final byte[] content;

    /**
     * Create an HTTP request
     * 
     * @param url
     *            the requested URL
     * @param method
     *            the HTTP method
     * @param content
     *            request's contend. May be null.
     * @param headers
     *            HTTP headers. May be null or empty.
     */
    public HTTPRequest(String url, HTTPMethod method, byte[] content, Map<String, String> headers)
    {
        this.url = Objects.requireNonNull(url, "url");
        this.method = Objects.requireNonNull(method, "method");
        this.content = content;
        this.headers = headers;
    }

    /**
     * Send the request.
     * 
     * @return the HTTP response
     * @throws IOException
     */
    public HTTPResponse invoke() throws IOException
    {
        URL target = new URL(url);
        HttpURLConnection connection = (HttpURLConnection) target.openConnection();
        connection.setRequestMethod(method.name());
        if (content != null)
        {
            connection.setRequestProperty(HTTPHeader.ContentLength, Integer.toString(content.length));
        }
        if (headers != null)
        {
            headers.entrySet().stream().forEach(e -> connection.setRequestProperty(e.getKey(), e.getValue()));
        }

        if (content != null)
        {
            try
            {
                connection.setDoOutput(true);
                connection.getOutputStream().write(content);
                connection.getOutputStream().flush();
            }
            finally
            {
                try
                {
                    connection.getOutputStream().close();
                }
                catch (Exception e)
                {
                }
            }
        }

        return new HTTPResponse(connection);
    }

    /**
     * @return the HTTP headers
     */
    public Map<String, String> getHeaders()
    {
        return headers;
    }

    /**
     * @return the content of the body of the request
     */
    public byte[] getContent()
    {
        return content;
    }

    /**
     * @return the target URL
     */
    public String getUrl()
    {
        return url;
    }

    /**
     * @return the HTTP method
     */
    public HTTPMethod getMethod()
    {
        return method;
    }

    @Override
    public String toString()
    {
        StringBuilder sb = new StringBuilder();
        sb.append(method);
        sb.append(" ");
        sb.append(url);
        if (content != null && content.length > 0)
        {
            sb.append(" -> ");
            sb.append(new String(content, 0, Math.min(content.length, 200)));
        }
        return sb.toString();
    }
}
