package nl.xelion.xcc.api.examples.client.http;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.reflect.Type;
import java.net.HttpURLConnection;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.google.gson.Gson;

/**
 * An HTTP Response
 * <p>
 * This software is provided under the MIT <a href="https://opensource.org/licenses/MIT">license</a>
 */
public class HTTPResponse
{
    private final int status;
    private final Map<String, List<String>> headers;
    private String content;

    public HTTPResponse(HttpURLConnection connection) throws IOException
    {
        status = connection.getResponseCode();
        headers = connection.getHeaderFields();
        content = readBody(connection);
    }

    /**
     * @return the returned HTTP status code
     */
    public int getStatus()
    {
        return status;
    }

    /**
     * @return the headers
     */
    public Map<String, List<String>> getHeaders()
    {
        return headers;
    }

    /**
     * @return the first value of the specified header
     */
    public String getHeaderValue(String name)
    {
        if (headers != null)
        {
            List<String> values = headers.get(name);
            if (values != null && !values.isEmpty())
            {
                return values.get(0);
            }
        }

        return null;
    }

    /**
     * @return the body content
     */
    public String getContent()
    {
        return content;
    }

    /**
     * @return the JSON object in the body content
     */
    public <T> T getJSONContent(Type type)
    {
        return new Gson().fromJson(content, type);
    }

    @Override
    public String toString()
    {
        StringBuilder sb = new StringBuilder("HTTP " + status);
        if (content != null && !content.isEmpty())
        {
            sb.append(" -> ");
            if (content.length() > 100)
            {
                sb.append(content.substring(0, 98) + "..");
            }
            else
            {
                sb.append(content);
            }
        }
        return sb.toString();
    }

    private static String readBody(HttpURLConnection connection) throws IOException
    {
        Charset charset = encodingCharset(connection.getContentEncoding());
        String input = null;
        try
        {
            input = readStream(connection.getInputStream(), charset);
        }
        catch (Exception e)
        {

        }
        String error = readStream(connection.getErrorStream(), charset);
        return input == null ? error : input;
    }

    private static String readStream(InputStream is, Charset charset)
    {
        if (is != null)
        {
            try
            {
                return new BufferedReader(new InputStreamReader(is, charset)).lines().collect(Collectors.joining("\n"));
            }
            catch (Exception e)
            {
            }
            finally
            {
                try
                {
                    is.close();
                }
                catch (Exception e)
                {
                }
            }
        }
        return null;
    }

    private static Charset encodingCharset(String encoding)
    {
        if (encoding != null && !encoding.isEmpty())
        {
            try
            {
                return Charset.forName(encoding);
            }
            catch (Exception e)
            {
            }
        }
        return StandardCharsets.UTF_8;
    }
}
